@@ -572,7 +572,7 @@ set_first_body_data(client_t *client, char *data, size_t datalen)
 static response_status
 write_headers(client_t *client, char *data, size_t datalen, char is_file)
 {
-    write_bucket *bucket; 
+    write_bucket *bucket = 0; 
     uint32_t hlen = 0;
     PyObject *headers = NULL, *templist = NULL;
     response_status ret;
@@ -583,7 +583,13 @@ write_headers(client_t *client, char *data, size_t datalen, char is_file)
     }
 
     //TODO ERROR CHECK
+    if (!client->headers){
+        goto error;
+    }
     headers = PySequence_Fast(client->headers, "header must be list");
+    if (!headers){
+        goto error;
+    }
     hlen = PySequence_Fast_GET_SIZE(headers);
 
     bucket = new_write_bucket(client->fd, (hlen * 4) + 42 );
@@ -616,7 +622,9 @@ write_headers(client_t *client, char *data, size_t datalen, char is_file)
         }
     }
 
-    if(client->keep_alive == 1){
+    if(client->status_code == 101){
+        add_header(bucket, "Connection", 10, "upgrade", 7);
+    }else if(client->keep_alive == 1){
         //Keep-Alive
         add_header(bucket, "Connection", 10, "Keep-Alive", 10);
     }else{
