@@ -244,6 +244,7 @@ new_client_t(int client_fd, char *remote_addr, uint32_t remote_port)
     //memset(client, 0, sizeof(client_t));
 
     client->fd = client_fd;
+    client->complete = 1;
     client->request_queue = new_request_queue();
     client->remote_addr = remote_addr;
     client->remote_port = remote_port;
@@ -515,7 +516,7 @@ static PyObject *
 app_handler(PyObject *self, PyObject *args)
 {
     int ret, active;
-    PyObject *wsgi_args = NULL, *start = NULL, *res = NULL;
+    PyObject *wsgi_args = NULL, *start = NULL, *current = NULL, *parent = NULL, *res = NULL;
     PyObject *env = NULL;
     ClientObject *pyclient;
     client_t *client;
@@ -572,6 +573,16 @@ app_handler(PyObject *self, PyObject *args)
             if ((ret == 0 && !active)) {
                 activecnt++;
             }
+
+            // switch to hub
+            current = pyclient->greenlet;
+            parent = greenlet_getparent(current);
+        
+            /* Py_INCREF(hub_switch_value); */
+            res = greenlet_switch(parent, hub_switch_value, NULL);
+            Py_XDECREF(res);
+
+            // try again after event switch
             status = process_body(client);
         }
     }
@@ -702,7 +713,6 @@ call_wsgi_handler(client_t *client)
 #ifdef WITH_GREENLET
     //new greenlet
     greenlet = greenlet_new(handler, NULL);
-    Py_DECREF(greenlet_getparent(greenlet));
     // set_greenlet
     pyclient->greenlet = greenlet;
     Py_INCREF(pyclient->greenlet);
@@ -1031,8 +1041,11 @@ set_read_error(client_t *client, int status_code)
         //finish = 1
         return 1;
     } else {
-        client->status_code = status_code;
-        send_error_page(client);
+        if (!client->complete) {
+            // read error while reading request.
+            client->status_code = status_code;
+            send_error_page(client);
+        } // else keepalive timeout. should not send any data.
         close_client(client);
         return -1;
     }
@@ -1865,7 +1878,7 @@ meinheld_run_loop(PyObject *self, PyObject *args, PyObject *kwds)
     }
 
     Py_DECREF(wsgi_app);
-    Py_XDECREF(watchdog);
+    Py_CLEAR(watchdog);
     
     current_client = NULL;
     picoev_destroy_loop(main_loop);
@@ -2033,8 +2046,9 @@ meinheld_set_watchdog(PyObject *self, PyObject *args)
         PyErr_SetString(PyExc_TypeError, "must be callable");
         return NULL;
     }
+    Py_INCREF(temp);
+    Py_XDECREF(watchdog);
     watchdog = temp;
-    Py_INCREF(watchdog);
     watch_loop = 1;
     Py_RETURN_NONE;
 }
@@ -2331,7 +2345,6 @@ meinheld_spawn(PyObject *self, PyObject *args, PyObject *kwargs)
     if (greenlet == NULL) {
         return NULL;
     }
-    Py_DECREF(greenlet_getparent(greenlet));
     res = internal_schedule_call(0, func, func_args, func_kwargs, greenlet);
     Py_XDECREF(res);
     DEBUG("greenlet refcnt:%d", (int)Py_REFCNT(greenlet));
